@@ -639,7 +639,7 @@ int test_ziplistStressWithRandomPayloadsOfDifferentEncoding(int argc, char **arg
     /* Hold temp vars from ziplist */
     unsigned char *sstr;
     unsigned int slen;
-    long long sval;
+    long long sval = 0;
 
     iteration = accurate ? 20000 : 20;
     for (i = 0; i < iteration; i++) {
