@@ -525,7 +525,7 @@ start_cluster 1 0 {tags {external:skip cluster}} {
         # +OK\r\n --> 5 bytes
 
         R 0 GET $key
-        # $3\r\nvalue\r\n -> 11 bytes
+        # $5\r\nvalue\r\n -> 11 bytes
 
         set expected_slot_stats [
             dict create $key_slot [
@@ -538,6 +538,27 @@ start_cluster 1 0 {tags {external:skip cluster}} {
     R 0 CONFIG RESETSTAT
     R 0 FLUSHALL
 
+    test "CLUSTER SLOT-STATS network-bytes-out, for slot specific commands, for reply copy avoidance" {
+        set copy_avoid [lindex [R 0 config get min-string-size-avoid-copy-reply] 1]
+        R 0 config set min-string-size-avoid-copy-reply 1
+
+        set value [string repeat A 1024] ;# Make sure it is a RAW string
+        R 0 set $key $value ;# +OK\r\n --> 5 bytes
+        R 0 get $key        ;# $1024\r\nAA..AA\r\n -> 1033 bytes
+
+        set expected_slot_stats [
+            dict create $key_slot [
+                dict create network-bytes-out 1038
+            ]
+        ]
+        set slot_stats [R 0 CLUSTER SLOT-STATS SLOTSRANGE $key_slot $key_slot]
+        assert_empty_slot_stats_with_exception $slot_stats $expected_slot_stats $metrics_to_assert
+
+        R 0 config set min-string-size-avoid-copy-reply $copy_avoid
+    }
+    R 0 CONFIG RESETSTAT
+    R 0 FLUSHALL
+
     test "CLUSTER SLOT-STATS network-bytes-out, blocking commands." {
         set rd [valkey_deferring_client]
         $rd BLPOP $key 0
@@ -975,3 +996,55 @@ start_cluster 1 1 {tags {external:skip cluster} overrides {cluster-slot-stats-en
     R 0 CONFIG RESETSTAT
     R 1 CONFIG RESETSTAT
 }
+
+start_cluster 1 0 {tags {external:skip cluster} overrides {cluster-slot-stats-enabled yes}} {
+    set key "testslotbytes"
+    set key_slot [R 0 cluster keyslot $key]
+    set metrics_to_assert [list network-bytes-out]
+
+    test "CLUSTER SLOT-STATS network-bytes-out with copy avoidance and commandlog disabled" {
+        set copy_avoid [lindex [R 0 config get min-string-size-avoid-copy-reply] 1]
+        R 0 config set min-string-size-avoid-copy-reply 1
+        
+        # Disable commandlog tracking
+        R 0 config set commandlog-reply-larger-than -1
+        R 0 config resetstat
+        
+        set value [string repeat A 2048]
+        R 0 set $key $value
+        
+        # Reset stats after SET to only measure GET reply
+        R 0 config resetstat
+        
+        # Get should use copy avoidance path
+        R 0 get $key
+        
+        # Verify cluster slot stats tracked the bytes correctly
+        # Even though commandlog tracking is disabled, cluster slot stats should work
+        # via IO thread accounting in releaseBufReferences()
+        set slot_stats [R 0 CLUSTER SLOT-STATS SLOTSRANGE 0 16383]
+        set slot_stats [convert_array_into_dict $slot_stats]
+        
+        set network_bytes_out [dict get $slot_stats $key_slot network-bytes-out]
+        
+        # For 2048 bytes: $2048\r\n<data>\r\n = 2057 bytes
+        assert_equal $network_bytes_out 2057
+        
+        # Re-enable commandlog
+        R 0 config set commandlog-reply-larger-than 1024
+        R 0 config resetstat
+        
+        # Get should still track correctly
+        R 0 get $key
+        
+        set slot_stats [R 0 CLUSTER SLOT-STATS SLOTSRANGE 0 16383]
+        set slot_stats [convert_array_into_dict $slot_stats]
+        set network_bytes_out [dict get $slot_stats $key_slot network-bytes-out]
+        
+        assert_equal $network_bytes_out 2057
+        
+        # Cleanup
+        R 0 del $key
+        R 0 config set min-string-size-avoid-copy-reply $copy_avoid
+    }
+}
